"""
This script is based on the LoadLeveler example program
/usr/lpp/LoadL/full/samples/lldata_access/wall_clock_time.c
"""

from pyloadl import *
import getopt, sys

StepState = ( "I", "P", "ST", "R",
    "CP", "XP", "RP",
    "VP", "C", "X", "RM",
    "V", "CA", "NR", "TX",
    "?", "SX", "H", "D",
    "NQ", "E", "EP",
    "MP")

optlist, args = getopt.getopt(sys.argv[1:], 'j:')
for o, a in optlist:
   if o in ("-j"):
      steplist = [a]

queryObject = ll_query(JOBS)
if not PyCObjValid(queryObject): 
  print "ll_query(): returns NULL\n"
  sys.exit(-1)

rc = ll_set_request(queryObject, QUERY_STEPID, steplist, ALL_DATA)
if rc: 
  print "ll_set_request(): QUERY_STEPID -  rc = %d\n" % rc
  sys.exit(-1)

job, obj_count, err_code = ll_get_objs(queryObject, LL_CM, "")
if err_code != 0:
  print "ll_get_objs(): error - %d\n" % err_code
  sys.exit(-1)

mach_name = ""
if job:

  step  = ll_get_data(job, LL_JobGetFirstStep)
  if step:

    wclh_req = ll_get_data(step, LL_StepWallClockLimitHard)
    wcls_req = ll_get_data(step, LL_StepWallClockLimitSoft)

    state = ll_get_data(step, LL_StepState)
    st = StepState[state]

    if ( state == STATE_RUNNING ) or ( state == STATE_PREEMPTED )\
       or ( state == STATE_PREEMPT_PENDING ) \
       or ( state == STATE_RESUME_PENDING ):

      machine  = ll_get_data(step, LL_StepGetFirstMachine)
      if machine:
         mach_name = ll_get_data(machine, LL_MachineName)
      else: 
         print "Error - unable to get machine name\n"

    else:
       print "Error - Jobstep not in appropiate state\n"

  else:
   print "Error - no jobstep information\n"

if mach_name:

  queryObject = ll_query(JOBS)
  rc = ll_set_request(queryObject, QUERY_STEPID, steplist, ALL_DATA)
  if rc:
    print "ll_set_request() - QUERY_STEPID -  RC = %d\n" % rc
    exit(1)

  job, obj_count, err_code = ll_get_objs(queryObject, LL_STARTD, mach_name)

  if job:
    step_obj = ll_get_data(job, LL_JobGetFirstStep)
    step_name = ll_get_data(step_obj, LL_StepID)
    step_wcl  = ll_get_data(step_obj, LL_StepWallClockUsed)
    print "WCLUSED - %d" % ll_get_data(step_obj, LL_StepWallClockUsed)

    remain_h = (wclh_req - step_wcl)
    remain_s = (wcls_req - step_wcl)
    print "                                             WallClock Remaining"
    print "   Hostname             StepId                Hard        Soft"
    print "--------------- ------------------------- ------------ ------------"

    print "%-15s %-25s %-10d %-10d " % (mach_name, step_name, remain_h, remain_s)
