from pyloadl import *
from time import gmtime, strftime
from sys import exit

def bgshape(bgshapearray):

  shape_string = ""
  for a in bgshapearray:

     if a > 0:
       if shape_string:
         shape_string = shape_string + "x" + str(a)
       else:
         shape_string = shape_string + str(a) 

  return shape_string

"""

   Displays information on the LoadLeveler queue 
   for BlueGene jobs similar to the llq -b command

"""

stepstate = [
    "I", "P", "ST", "R",
    "CP", "XP", "RP",
    "VP", "C", "X", "RM",
    "V", "CA", "NR", "TX",
    "?", "SX", "H", "D",
    "NQ", "E", "EP",
    "MP" ]

bgwire = [ "MESH","TORUS", "BG_NAV", "PREFER_TORUS" ]
HoldType = ["", "H", "S", "HS"]

jobs_total = 0
jobs_run = 0
jobs_idle = 0
jobs_pending = 0
jobs_held = 0
jobs_preempt = 0

# Query Job information

query = ll_query(JOBS)
if not PyCObjValid(query):
   print "ll_query(): returns NULL\n" 
   exit(-1)

retval = ll_set_request(query, QUERY_BG_JOB, "", ALL_DATA)
if retval != 0:
   print "ll_set_request(): error %d\n" % retval
   exit(-1)

job, num, err = ll_get_objs(query, LL_CM, "")
if err != 0:
   print "ll_get_objs(): error %d" % err
   exit(-1)

if num > 0:

   while PyCObjValid(job):

      cred  = ll_get_data(job,  LL_JobCredential)
      owner = ll_get_data(cred, LL_CredentialUserName)

      sdate = ll_get_data(job, LL_JobSubmitTime)
      date = ""
      if sdate != 0:
        sdate = gmtime(sdate)
        date  = strftime("%m/%d %H:%M", sdate)

      # Loop through all steps for this job

      step = ll_get_data(job, LL_JobGetFirstStep)
      while PyCObjValid(step):

        #           Step Type: Blue Gene
        #      Size Requested:
        #      Size Allocated: 128
        #     Shape Requested:
        #     Shape Allocated:
        #    Wiring Requested: MESH
        #    Wiring Allocated: MESH
        #              Rotate: True
        #    Blue Gene Status:
        #    Blue Gene Job Id:
        # Partition Requested: bot128-1

        steptype = ll_get_data(step, LL_JobStepType)
        bgsizereq= ll_get_data(step, LL_StepBgSizeRequested)
        bgsizeall= ll_get_data(step, LL_StepBgSizeAllocated)
        bgshpreq = ll_get_data(step, LL_StepBgShapeRequested)
        bgshpall = ll_get_data(step, LL_StepBgShapeAllocated)
        bgwirereq= ll_get_data(step, LL_StepBgConnectionRequested)
        bgwireall= ll_get_data(step, LL_StepBgConnectionAllocated)
        partreq  = ll_get_data(step, LL_StepBgPartitionRequested)
        partall  = ll_get_data(step, LL_StepBgPartitionAllocated)
        bgstatus = ll_get_data(step, LL_StepBgJobState)
        bgjobid  = ll_get_data(step, LL_StepBgJobId)

        print "\tStep Type: %s" % steptype
        print "\tSize Requested: %s" % bgsizereq
        print "\tSize Allocated: %s" % bgsizeall
        bgshpall = bgshape(bgshpall)
        bgshpreq = bgshape(bgshpreq)
        print "\tShape Requested: %s" % bgshpreq
        print "\tShape Allocated: %s" % bgshpall
        bgwirereq = bgwire[bgwirereq]
        bgwireall = bgwire[bgwireall]
        print "\tWiring Requested: %s" % bgwirereq
        print "\tWiring Allocated: %s" % bgwireall
        print "\tBlue Gene Status: %s" % bgstatus
        print "\tBlue Gene Job Id: %s" % bgjobid
        print "\tPartition Requested: %s" % partreq
 
        tpn  = ll_get_data(step, LL_StepTasksPerNodeRequested)
        state  = ll_get_data(step, LL_StepState)
        status = stepstate[state]

        machine = ""
        if (state == STATE_RUNNING):
          machine_obj = ll_get_data(step, LL_StepGetFirstMachine)
          if PyCObjValid(machine_obj):
             machine = ll_get_data(machine_obj, LL_MachineName)
        else:
          holdtype = ll_get_data(step, LL_StepHoldType)
          if (holdtype > 0 ):
            status = HoldType[holdtype]
            jobs_held = jobs_held + 1

        id     = ll_get_data(step, LL_StepID)
        pri    = ll_get_data(step, LL_StepPriority)
        jclass = ll_get_data(step, LL_StepJobClass)

        step = ll_get_data(job, LL_JobGetNextStep)

      job = ll_next_obj(query)

# Free and deallocate objects used by LoadLeveler

ll_free_objs(query)
ll_deallocate(query)

