from pyloadl import *
from sys import exit
from string import join

"""

   Displays information on the LoadLeveler multi-cluster

"""

# Query MultiClusters

query = ll_query(MCLUSTERS)
if not PyCObjValid(query):
   print "ll_query(): returns NULL\n"
   exit(-1)

retval = ll_set_request(query, QUERY_ALL, "", ALL_DATA)
if retval != 0:
   print "ll_set_request(): error %d\n" % retval
   exit(-1)

cluster, num, err = ll_get_objs(query, LL_SCHEDD, "")
if err != 0:
   print "ll_get_objs(): error %d" % err
   exit(-1)

cluster_list = []
while PyCObjValid(cluster):

  cluster_name = ll_get_data(cluster, LL_MClusterName)
  cluster_list.append(cluster_name)

  cluster = ll_next_obj(query)

# Free and deallocate objects used by LoadLeveler

ll_free_objs(query)
ll_deallocate(query)

# Query information on each cluster

for cluster_name in cluster_list:

  isdata = 1
  rc, error = ll_cluster([cluster_name], CLUSTER_SET)
  if err != 0:
     print "ll_cluster(): cluster name %s error %d" % (cluster_name, error)

  print "=================== Cluster %s ====================================\n" % cluster_name

  query = ll_query(CLUSTERS)
  if not PyCObjValid(query):
     print "ll_query(): returns NULL\n"
     isdata = 0

  b = ll_set_request(query, QUERY_ALL, [""], ALL_DATA)
  if b != 0:
     print  "ll_set_request(): error %d\n" % b
     isdata = 0

  cluster, num, error = ll_get_objs(query, LL_CM, "")
  if error != 0:
     print  "ll_get_objs(): error %d\n" % error
     isdata = 0

  if isdata:

    while PyCObjValid(cluster):
  
      schedtype = ll_get_data(cluster, LL_ClusterSchedulerType)
      print "Scheduler Type          : %s" %  schedtype

      enfsub = ll_get_data(cluster, LL_ClusterEnforceSubmission)
      print "Enforced Submission     : %s\n" % bool(enfsub)

      enfmem = ll_get_data(cluster, LL_ClusterEnforceMemory)
      print "Enforced Memory         : %s\n" % bool(enfmem)

      enfrescnt = ll_get_data(cluster, LL_ClusterEnforcedResourceCount)
      print "Enforced Resource Count : %d" % enfrescnt
      enfres = ll_get_data(cluster, LL_ClusterEnforcedResources)
      print "Enforced Resources      : %s\n" % join(enfres,", ")

      defrescnt = ll_get_data(cluster,LL_ClusterDefinedResourceCount)
      print "Defined Resource Count  : %d" % defrescnt
      defres = ll_get_data(cluster,LL_ClusterDefinedResources)
      print "Defined Resources       : %s\n" % join(defres,", ")

      schedrescnt = ll_get_data(cluster,LL_ClusterSchedulingResourceCount)
      print "Scheduled Resource Count: %d" % schedrescnt
      schedres = ll_get_data(cluster, LL_ClusterSchedulingResources)
      print "Scheduled Resources     : %s\n" % join(schedres,", ")

      print "Resource Info :\n"
      resource = ll_get_data(cluster, LL_ClusterGetFirstResource)
      while PyCObjValid(resource):

        name = ll_get_data(resource, LL_ResourceName)
        init_val = ll_get_data(resource, LL_ResourceInitialValue)
        avail_val = ll_get_data(resource, LL_ResourceAvailableValue)
        print "Name: %s, Inital : %d, Available : %d" % (name, init_val, avail_val)
        resource = ll_get_data(cluster, LL_ClusterGetNextResource)

      print ""    
      cluster = ll_next_obj(query)

    #
    # Free up space allocated by LoadLeveler
    #

    ll_free_objs(query)
    ll_deallocate(query)

    rc, error = ll_cluster([cluster_name], CLUSTER_UNSET)
    if err != 0:
       print "ll_cluster(): unset %s error %s" % (cluster_name,error)

