from pyloadl import *
from optparse import OptionParser
from time import gmtime, strftime
from sys import exit

"""

   Displays information on fairshare, 
   similar to the llfs LoadLeveler program

"""

parser = OptionParser()
usage = "usage: %prog [ -h | -s savedir | -r [savedfile] | [ -u userlist ] [ -g grouplist ] ]"
parser = OptionParser(usage=usage)

parser.add_option("-s", action="store", type="string", dest="saved_dir",
                 default="",help="Directory to save fairshare data to", metavar="/path")
parser.add_option("-r", action="store", type="string", dest="saved_file",
                 default="", help="File to restore fairshare data from", metavar="/path/file")
parser.add_option("-u", action="store", dest="users",
                 help="Comma delimited list of fairshare users", metavar="user,user .....")
parser.add_option("-g", action="store", dest="groups",
                  help="Comma delimited list of fairshare groups", metavar="group,group .....")

(options, args) = parser.parse_args()
if ( not options.saved_dir and not options.saved_file and not options.users and not options.groups ):
   print "Usage: llfs.py [ -h | -s savedir | -r [savedfile] | [ -u userlist ] [ -g grouplist ] ]"
   exit(-1)

if ( options.saved_dir or options.saved_file ) and ( options.users or options.groups ):
   print "Usage: llfs.py [ -h | -s savedir | -r [savedfile] | [ -u userlist ] [ -g grouplist ] ]"
   exit(-1)

# Save/Restore Fair Share Data

if options.saved_dir and options.saved_file:
   parser.error("options -s and -r are mutually exclusive")
else:
   if options.saved_dir != "" or options.saved_file != "":

     if options.saved_file != "":
        operation = FAIR_SHARE_RESET
     elif options.saved_dir != "":
        operation = FAIR_SHARE_SAVE
     else:
        parser.error("Must provide parameter for chosen option")
        exit(-1)

     (rc, errObj) = ll_fair_share(operation, options.saved_dir, options.saved_file)

     ll_error(errObj, 1)
     exit(rc)

# Query Job information

query = ll_query(FAIRSHARE)
if not PyCObjValid(query):
   print "ll_query(FAIRSHARE): returns NULL\n" 
   exit(-1)

if ( len(options.users) == 0 and len(options.groups) == 0 ):
   retval = ll_set_request(query, QUERY_ALL, "", ALL_DATA)
elif (options.users):
   users = string.split(options.users, ",")
   retval = ll_set_request(query, QUERY_USER, users, ALL_DATA)
elif (groups):
   groups = string.split(options.groups, ",")
   retval = ll_set_request(query, QUERY_GROUP, groups, ALL_DATA)

if retval != 0:
   print "ll_set_request(): error %d\n" % retval
   exit(-1)

fair_share, num, err_code = ll_get_objs(query, LL_CM, "")
if err_code != 0:
   print "ll_get_objs(): error %d" % err
   exit(-1)

if num > 0:

   if PyCObjValid(fair_share):

      current_time = ll_get_data(fair_share, LL_FairShareCurrentTime)
      date = ""
      if current_time != 0:
        sdate = gmtime(current_time)
        date  = strftime("%m/%d %H:%M", sdate)

      interval    = ll_get_data(fair_share, LL_FairShareInterval)
      total_share = ll_get_data(fair_share, LL_FairShareTotalShares)

      entries      = ll_get_data(fair_share, LL_FairShareNumberOfEntries)
      entry_names  = ll_get_data(fair_share, LL_FairShareEntryNames)
      entry_types  = ll_get_data(fair_share, LL_FairShareEntryTypes)
      alloc_shares = ll_get_data(fair_share, LL_FairShareAllocatedShares)
      used_shares  = ll_get_data(fair_share, LL_FairShareUsedShares)

      print "Current Time %s" % date
      print "FAIR_SHARE_TOTAL_SHARES %s" % total_share
      print "FAIR_SHARE_INTERVAL %s" % interval

      print "\nName                 Type  Allocated Shares Used Shares"
      print "-------------------- ----- ---------------- -----------"

      i = 0
      for i in range(entries):
         print "%-20s %-5s %-16d %d" % (entry_names[i], entry_types[i], alloc_shares[i], used_shares[i])

# Free and deallocate objects used by LoadLeveler

ll_free_objs(query)
ll_deallocate(query)

