from pyloadl import *
from time import gmtime, strftime
from sys import exit

"""

   Displays information on the LoadLeveler queue 
   similar to the llq command

"""

stepstate = [
    "I", "P", "ST", "R",
    "CP", "XP", "RP",
    "VP", "C", "X", "RM",
    "V", "CA", "NR", "TX",
    "?", "SX", "H", "D",
    "NQ", "E", "EP",
    "MP" ]

HoldType = ["", "H", "S", "HS"]

jobs_total = 0
jobs_run = 0
jobs_idle = 0
jobs_pending = 0
jobs_held = 0
jobs_preempt = 0

# Query Job information

query = ll_query(JOBS)
if not PyCObjValid(query):
   print "ll_query(): returns NULL\n" 
   exit(-1)

retval = ll_set_request(query, QUERY_TOP_DOG, "", ALL_DATA)
if retval != 0:
   print "ll_set_request(): error %d\n" % retval
   exit(-1)

job, num, err = ll_get_objs(query, LL_CM, "")
if ( not PyCObjValid(job) or num == 0 ):
   print "There are no top dogs at this time."
   exit(0)

if err != 0:
   print "ll_get_objs(): error %d" % err
   exit(-1)

if num > 0:
   print "Step                               Owner      q_sysprio  Estimated Start Time"
   print "---------------------------------- ---------- ---------- ------------------------"

   while PyCObjValid(job):

      cred  = ll_get_data(job,  LL_JobCredential)
      owner = ll_get_data(cred, LL_CredentialUserName)

      step = ll_get_data(job, LL_JobGetFirstStep)
      while PyCObjValid(step):

         step_id     = ll_get_data(step, LL_StepID)
         q_sysprio   = ll_get_data(step, LL_StepQueueSystemPriority)
         step_cdate  = ll_get_data(step, LL_StepConsideredAt)
         if step_cdate:
           cdate = gmtime(step_cdate)
           cdate = strftime("%a %b %d %H:%M:%S %Y", cdate)

         step_estart = ll_get_data(step, LL_StepEstimatedStartTime)
         if step_estart:
           edate = gmtime(step_estart)
           edate = strftime("%a %b %d %H:%M:%S %Y", edate)

         is_top_dog = ll_get_data(step, LL_StepIsTopDog)
         if is_top_dog:
            print "%-34s %-10s %-10d %s" % (step_id, owner, q_sysprio, edate)
         else:
            print "The step %s is not a top dog? Unbelievable!" % step_id

         step = ll_get_data(job, LL_JobGetNextStep)

      job = ll_next_obj(query)

   print "========= The top dogs were considered for scheduling at %s" % cdate

   ll_free_objs(query)
   ll_deallocate(query)
