"""
 This is a Python version of the example that comes with LoadLeveler in
 LoadL/full/samples/lldata_access/machine_job_query.c

 ---------------------------------------------------------------------------------
 This sample program shows how LoadLeveler's Data Access API can be used
 to obtain machine, job, and cluster information. The program consists of
 three steps:
   1. Get information of selected machines in the LoadLeveler cluster.
   2. Get information of jobs of selected classes.
   3. Get floating consumable resource information of the LoadLeveler cluster.

 Notes:
   1. The host_list of machine names must be modified to match your configuration.
   2. The class_list of class names must be modified to match your configuration.
 ---------------------------------------------------------------------------------
"""

from pyloadl import *
from sys import exit

# Step 1: Display information of selected machines in the LL cluster

#Initialize the query: Machine query

queryObject = ll_query(MACHINES)
if not PyCObjValid(queryObject):
    print "Query MACHINES: ll_query() returns NULL."
    exit(-1)

# Set query parameters: query specific machines by name

rc = ll_set_request(queryObject, QUERY_HOST, ["shivling"], ALL_DATA)
if rc:
    print "Query MACHINES: ll_set_request() return code %d" % rc 
    exit(-1)

# Get the machine objects from the LoadL_negotiator (central manager) daemon

machine, obj_count, err_code = ll_get_objs(queryObject, LL_CM, "")
if err_code != 0:
    print "Query MACHINES: ll_get_objs() returns NULL. Error code =  %d" % err_code
    exit(-1)

# Process the machine objects

while PyCObjValid(machine):

    name = ll_get_data(machine, LL_MachineName)
    if (name):
	print "------------------ %s ------------------" % name

    startd_state = ll_get_data(machine, LL_MachineStartdState)
    if (not startd_state):
	print "Query MACHINES: ll_get_data() return code is non-zero."
	exit(1)

    print "CPUS : %s" % ll_get_data(machine, LL_MachineCPUs)
    print "CPU List: %s" % ll_get_data(machine, LL_MachineCPUList)
    print "Used CPUS : %s" % ll_get_data(machine, LL_MachineUsedCPUs)
    print "Used CPU List: %s" % ll_get_data(machine, LL_MachineUsedCPUList)
    if startd_state != "Down":
         value64 = ll_get_data(machine, LL_MachineRealMemory64)
         print "Total Real Memory: %d MB" % value64
         value64 = ll_get_data(machine, LL_MachineVirtualMemory64)
         print "Free Swap Space: %d KB" % value64
         load_avg = ll_get_data(machine, LL_MachineLoadAverage)
         print "Load Average: %f" % load_avg

    # Consumable Resources associated with this machine

    resource = ll_get_data(machine, LL_MachineGetFirstResource)
    while PyCObjValid(resource):
	res_name = ll_get_data(resource, LL_ResourceName)
	print "Resource Name = %s" % res_name
	value64 =  ll_get_data(resource, LL_ResourceInitialValue64)
	print "   Total: %d" % value64
	value64 = ll_get_data(resource, LL_ResourceAvailableValue64)
	print "   Available: %d " % value64
	resource = ll_get_data(machine, LL_MachineGetNextResource)

    machine = ll_next_obj(queryObject)

# Free objects obtained from Negotiator

ll_free_objs(queryObject)
ll_deallocate(queryObject)

#Step 2: Display information of selected jobs

#Initialize the query: Job query

print "=================================================================="

queryObject = ll_query(JOBS)
if not queryObject:
  print "Query JOBS: ll_query() returns NULL."
  exit(-1)

rc = ll_set_request(queryObject, QUERY_HOST, ["shivling"], ALL_DATA)
if rc:
  print "Query JOBS: ll_set_request() return code is %d" % rc
  exit(-1)

rc = ll_set_request(queryObject, QUERY_CLASS, ["small", "large"], ALL_DATA)
if rc: 
  print "Query JOBS: ll_set_request() return code is %d" % rc
  exit(-1)

# Get the requested job objects from the Central Manager
job, obj_count, err_code = ll_get_objs(queryObject, LL_CM, "")
if not job:
  print "Query JOBS: ll_get_objs() returns NULL. Error code = %d" % err_code
  exit(-1)

#
# Process the job objects and display selected information of each job step.
#
#  Notes:
#    1. Since LL_element is defined as "void" in llapi.h, when using
#       ll_get_data it is important that a valid "specification" parameter
#       be used for a given "element" argument.
#    2. Checking of return code is not always made in the following loop to
#       minimize the length of the listing.
#

while PyCObjValid(job):

  print "------------------------------------------------------------------"
  name = ll_get_data(job, LL_JobName)
  print "Job name: name"

  credential = ll_get_data(job, LL_JobCredential)
  name = ll_get_data(credential, LL_CredentialUserName)
  print "Job owner: %s" % name
  name = ll_get_data(credential, LL_CredentialGroupName)
  print "Unix Group: %s" % name

  step = ll_get_data(job, LL_JobGetFirstStep)
  while PyCObjValid(step):

    print "  ----------------------------------------------------------------"
    step_id = ll_get_data(step, LL_StepID)
    print "  Step ID: %s" % step_id
    job_class = ll_get_data(step, LL_StepJobClass)
    print "  Step Job Class: %s" % job_class
    value64 = ll_get_data(step, LL_StepCpuStepLimitHard64)
    print "  Job Step CPU Hard Limit: %d" % value64
    value64 = ll_get_data(step, LL_StepCpuLimitHard64)
    print "  Step CPU Hard Limit: %d" % value64
    value64 = ll_get_data(step, LL_StepFileLimitHard64)
    print "  Step File Hard Limit: %d" % value64
    step_state = ll_get_data(step, LL_StepState)

    if step_state == STATE_RUNNING:
      print "  Step Status: Running"
      print "  Allocated Hosts:"
      machine = ll_get_data(step, LL_StepGetFirstMachine)

      while PyCObjValid(machine):
        name = ll_get_data(machine, LL_MachineName)
        print "    %s" % name
        machine = ll_get_data(step, LL_StepGetNextMachine)

    else:
      print "  Step Status: Not Running"
           
    node = ll_get_data(step, LL_StepGetFirstNode)
    while PyCObjValid(node):
      node_req = ll_get_data(node, LL_NodeRequirements)
      print "    Node Requirements: %s" % node_req

      task = ll_get_data(node, LL_NodeGetFirstTask)
      while PyCObjValid(task):

        task_exec = ll_get_data(task, LL_TaskExecutable)
        print "      Task Executable: %s" % task_exec
        ex_args = ll_get_data(task, LL_TaskExecutableArguments)
        print "      Task Executable Arguments: %s" % ex_args

        resource_req = ll_get_data(task, LL_TaskGetFirstResourceRequirement)
        while PyCObjValid(resource_req):

          name = ll_get_data(resource_req, LL_ResourceRequirementName)
          print "        Resource Req Name: %s" % name
          value64 =  ll_get_data(resource_req, LL_ResourceRequirementValue64)
          print "        Resource Req Value: %d" % value64
          resource_req = ll_get_data(task, LL_TaskGetNextResourceRequirement)
		 
        task = ll_get_data(node, LL_NodeGetNextTask)
	     
      node = ll_get_data(step, LL_StepGetNextNode)
	 
    step = ll_get_data(job, LL_JobGetNextStep)
      
  job = ll_next_obj(queryObject)
   
ll_free_objs(queryObject)
ll_deallocate(queryObject)

# Step 3: Display Floating Consumable Resources information of LL cluster.

# Initialize the query: Cluster query

print "=================================================================="
queryObject = ll_query(CLUSTERS)
if not queryObject:
  print "Query CLUSTERS: ll_query() returns NULL."
  exit(-1)

rc = ll_set_request(queryObject, QUERY_ALL, [""], ALL_DATA)
if rc:
  print "Query CLUSTERS: ll_set_request() returns a non zero."
  exit(-1)

cluster, obj_count, err_code = ll_get_objs(queryObject, LL_CM, "")
if err_code != 0:
  print "Query CLUSTERS: ll_get_objs() returns NULL. Error code = %d" % err_code
  exit(-1)

while PyCObjValid(cluster):

  resource = ll_get_data(cluster, LL_ClusterGetFirstResource)
  while PyCObjValid(resource):

    name = ll_get_data(resource, LL_ResourceName)
    init_val = ll_get_data(resource, LL_ResourceInitialValue64)
    avail_val = ll_get_data(resource, LL_ResourceAvailableValue64)
    print "Name: %s, Inital : %d, Available : %d" % (name, init_val, avail_val)
    resource = ll_get_data(cluster, LL_ClusterGetNextResource)
   
  cluster = ll_next_obj(queryObject)

ll_free_objs(queryObject)
ll_deallocate(queryObject)

