
# Sample raw RSS file generator, this works quite nicely
# with the KNewsticker in KDE (www.kde.org) - for which I 
# generally found best to set the number of recent items 
# to 1 for the purpose of this feed.

from pyloadl import *
from sys import exit 

StepState = ( "I", "P", "ST", "R",
    "CP", "XP", "RP",
    "VP", "C", "X", "RM",
    "V", "CA", "NR", "TX",
    "?", "SX", "H", "D",
    "NQ", "E", "EP",
    "MP" )

jobs_idle = 0
jobs_run  = 0
jobs_held = 0
jobs_not_queued = 0

# Query Job Status information

query = ll_query(JOBS)
if not PyCObjValid( query ):
  print "ll_query(): returns NULL\n"
  exit(-1)

rc = ll_set_request(query, QUERY_ALL, "", ALL_DATA)
if ( rc != 0 ):
  print "ll_set_request failed Return = %d" % rc

# Query the scheduler for information

job, number, err = ll_get_objs(query, LL_CM, "")
while PyCObjValid( job ):

  # Get the status for the first step of that Job

  step = ll_get_data(job, LL_JobGetFirstStep)
  holdtype = ll_get_data(step, LL_StepHoldType)
  state = ll_get_data(step, LL_StepState)
  st = StepState[state]

  if ( st == "R" ):   jobs_run = jobs_run + 1
  if ( st == "I" ):   jobs_idle = jobs_idle + 1
  if ( st == "NQ" ):  jobs_not_queued = jobs_not_queued + 1
  if ( hold_type > 0 ):
    jobs_held = jobs_held + 1

  job = ll_next_obj(query)

# Free up space allocated by LoadLeveler

ll_free_objs(job)
ll_deallocate(query)

title_string = "Running = %s, Idle = %s, Held = %s, NotQueued = %s" % (jobs_run, jobs_idle, jobs_held, jobs_not_queued)

nodes_busy = 0
nodes_idle = 0
nodes_running = 0
nodes_down = 0

# Query Node Status information

query = ll_query(MACHINES)
rc = ll_set_request(query, QUERY_ALL, "", ALL_DATA)

if (rc != 0 ):
  print "ll_set_request failed Return = %d" % rc

# Query the central manager for information
# number will contain the number of objects returned

machine, number, err = ll_get_objs(query, LL_CM, "")
while PyCObjValid( machine ):
   
  # Get the startd status for each machine

  sts = ll_get_data(machine, LL_MachineStartdState)

  if ( sts == "Busy" ):      nodes_busy = nodes_busy + 1
  if ( sts == "Idle" ):      nodes_idle = nodes_idle + 1
  if ( sts == "Running" ):   nodes_running = nodes_running + 1
  if ( sts == "Down" ):      nodes_down = nodes_down + 1

  machine = ll_next_obj(query)

# Free up space allocated by LoadLeveler

ll_free_objs(machine)
ll_deallocate(query)

node_string = "Busy = %s, Running = %s, Idle = %s, Down = %s" % (nodes_busy, nodes_running, nodes_idle, nodes_down)

# Write an RSS version 2 file

RDF = open("/srv/www/htdocs/loadl.xml","w")
RDF.write("<?xml version=\"1.0\" encoding=\"ISO-8859-1\" ?>\n")
RDF.write("<rss version=\"2.0\">\n")
RDF.write("   <channel>\n")
RDF.write("     <title>Makalu | LoadLeveler Status</title>\n")
RDF.write("     <language>en-gb</language>\n")
RDF.write("     <generator>PyLoadL</generator> 
RDF.write("     <ttl>5</ttl>\n")
RDF.write("     <item>\n")
RDF.write("       <title>Job Info</title>\n")
RDF.write("       <description>%s</description>\n" % title_string)
RDF.write("     </item>\n")
RDF.write("     <item>\n")
RDF.write("       <title>Node Status</title>\n")
RDF.write("       <description>%s</description>\n" % node_string)
RDF.write("     </item>\n")
RDF.write("   </channel>\n")
RDF.write("</rss>\n")
RDF.close()
