"""

PyLoadL: Python interface to IBM LoadLeveler

PyLoadL is a Pyrex wrapper that provides an interface 
to the IBM LoadLeveler TWS batch scheduler API

"""

from distutils.core import setup
from distutils.extension import Extension
from distutils.command import clean
from distutils.sysconfig import get_python_lib
from Pyrex.Distutils import build_ext

import os, re
import sys, platform
from string import *
from stat import *

VERSION = "1.0.0"
print "-------------- Python LoadLeveler %s Setup -----------------\n" % VERSION

print "\nNote :\tIf there are warnings regarding using incompatible libraries"
print "\tthen it is likely that you are mixing 32bit and 64bit, you"
print "\tshould change the library_dirs list in setup.py\n"

# Mininmum of Python 2.3 required because that's what Pyrex 0.9.5.1+ requires

if not hasattr(sys, 'version_info') or sys.version_info < (2,3,0, 'final'):
   raise SystemExit, "Python 2.3.0 or later required to build Pyloadl."
else:
   print "Detected Python %s.%s.%s in use" % sys.version_info[:3]

OS = platform.system()
if OS == "AIX":

  print "Building for AIX"

  # Standard LoadLeveler 4.x location under AIX

  include_dirs = ['/usr/lpp/LoadL/scheduler/full/include', '/usr/include']
  library_dirs = ['/usr/lpp/LoadL/scheduler/full/lib', '/usr/lib']
  libraries = ['llapi']

  llctl_dir = '/usr/lpp/LoadL/full/bin/'

elif OS == "Linux":

  print "Building for linux"

  # Standard LoadLeveler 4.x location under Linux

  include_dirs = ['/opt/ibmll/LoadL/scheduler/full/include', '/usr/include']
  library_dirs = ['/opt/ibmll/LoadL/scheduler/full/lib', '/usr/lib64', '/usr/lib']
  libraries = ['llapi']

  llctl_dir = '/opt/ibmll/LoadL/full/bin/'

else:
  print "Sorry but %s is not a valid OS....exiting" % OS
  sys.exit(1)

#
# Get the output from LoadLeveler command llctl
#

if sys.version_info < (2, 4):
  import popen2
  pout, pin, perr = popen2.popen3('%s/llctl -v' % llctl_dir,)
  pin.close()
  a = pout.readline()
  pout.close()
  b = perr.readline()
  perr.close()
else:
  import subprocess
  cmd_pipe = subprocess.Popen('%s/llctl -v' % llctl_dir, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, close_fds=True)
  cmd_pipe.wait()
  a, b = cmd_pipe.communicate()

if (b):
  print ""
  print "WARNING: Error detected querying LoadLeveler command llctl -v\n"
  print "%s" % b
  print "Exiting ...."
  sys.exit(1)

#
# Search llctl output for version string
#

LLCTL = re.compile(r'^llctl\s+(?P<VERSION>\d+\.\d+\.\d+\.\d+).*')

# If the LoadLeveler version is incorrectly detected then
# set the appropiate version in LLVER i.e '03030101' = 3.3.1.1
# by hand to force the correct version

LLVER = ''
if ( not LLVER ):

  if a[-1] == '\n': a = a[:-1]

  line = LLCTL.match(a)
  if line:

    if line.group("VERSION"):

      version = line.group("VERSION")
      ver_str = version.split('.')

# For creating LoadLeveler version dependent PyLoadL
# code create a version string similar to the
# LoadLeveler PERL module alla Mike Hawkins
# Pad the version string so 3.3.1.11 = '3030111'

  LLVER = LLVER + ver_str[0]
  for a in ver_str[1:]:
     if len(a) < 2:
       a = '0' + a
     LLVER = LLVER + a

if LLVER < '4010000':
    print "WARNING: This module only supports LoadLeveler 4.1 or higher\n"
    print "Exiting ...."
    sys.exit(1)
else:
    print "Detected LoadLeveler version %s" % LLVER

compiler_dir = os.path.join(get_python_lib(prefix=''), 'pyloadl/')

# Run the template files through the C preprocessor
# Change the CPP definition here to suit your C preprocessor

if OS == "Linux":
  CPP = "cpp -D__linux__ -E"  # gcc not vacpp
  #CPP = "cc -D__linux__ -E"  # vac (IBM)
else:
  CPP = "cc -E" # AIX XLC

os.system("%s -DLLVER=%s pyloadl/pyloadl.pyx.tmpl | egrep -v '^# |#line|# line|# 1|##' > pyloadl/pyloadl.pyx" % (CPP, LLVER))
os.system("%s -DLLVER=%s pyloadl/pyloadlvar.py.tmpl | egrep -v '#line|# line|# 1|##' > pyloadl/pyloadlvar.py" % (CPP, LLVER))

# Trove classifiers

classifiers = """\
Development Status :: 4 - Beta
Intended Audience :: Developers
License :: OSI Approved :: GNU General Public License (GPL)
Natural Language :: English
Operating System :: POSIX :: AIX :: Linux
Programming Language :: Python
Topic :: Software Development :: Libraries :: Python Modules
"""

# Disutils fix for Python versions < 2.3 that didn't
# support classifiers as listed at
# http://www.python.org/~jeremy/weblog/030924.html

if sys.version_info < (2, 3):
    _setup = setup
    def setup(**kwargs):
       if kwargs.has_key("classifiers"):
          del kwargs["classifiers"]
       _setup(**kwargs)

doclines = __doc__.split("\n")

setup(
    name = "PyLoadL",
    version = "1.0.0",
    description = doclines[0],
    long_description = "\n".join(doclines[2:]),
    author = "Mark Roberts",
    author_email = "mark at gingergeeks co uk",
    url = "http://www.gingergeeks.co.uk/pyloadl/",
    classifiers = filter(None, classifiers.split("\n")),
    platforms = ["Linux", "AIX"],
    keywords = ["IBM", "LoadLeveler"],
    packages = ["pyloadl"],
    ext_modules = [
        Extension( "pyloadl/pyloadl",["pyloadl/pyloadl.pyx"],
                   libraries = libraries,
                   library_dirs = library_dirs,
                   include_dirs = include_dirs )
    ],
    cmdclass = {"build_ext": build_ext}
)
